/* libgraf.h */

/* Definicions de constants */

#define MAXVER 100


/* Definicions de tipus */

typedef enum {FALS = 0, CERT = 1} boolea;


/* Punts i vectors */

typedef struct {
	float x, y;
	} Pt2D;

typedef struct {
	int x, y;
	} PtDisp;


/* Poligons */

typedef struct {
	int nv;
	Pt2D v[MAXVER];
	} Pol2D;

typedef struct {
	Pt2D vmin, vmax;
	} Capsa2D;

typedef struct {
	PtDisp vmin, vmax;
	} CapsaDisp;


/* Transformacions geomtriques 2D */

typedef float mat3x3[3][3];
typedef float vector3[3];

/* Procs de visualitzaci 2D */

typedef CapsaDisp Viewport;
typedef Capsa2D   Window;

/* Capaleres de procediments */

/* Poligons */

Capsa2D CapsaMinContPol2D (Pol2D pol);
/* Crea la capsa mnima contenidora del polgon */

boolea PtIntPol2D (Pt2D p, Pol2D pol);
/* Retorna si el punt s interior al polgon */

boolea PtIntCapsa2D (Pt2D p, Capsa2D c);
/* Retorna si el punt s interior a la capsa */

boolea PtIntCapsaDisp (PtDisp p, CapsaDisp c);
/* Retorna si el punt de dispositiu s interior a la capsa */

/* Transformacions geomtriques 2D */

void Identitat3x3 (mat3x3 M);
/* Crea la matriu identitat de 3x3 */

void CreaMatTranslacio2D (float tx, float ty, mat3x3 M);
/* Crea una matriu de translaci del vector (tx, ty) */

void CreaMatEscalat2D (float sx, float sy, mat3x3 M);
/* Crea una matriu d'escalat de factors sx i sy respecte l'origen */

void ConcatMat3x3 (mat3x3 M1, mat3x3 M2);
/* calcula M1 x M2 i volca el resultat a M1 */

void ProdMatVect3 (vector3 v, mat3x3 M, vector3 w);
/* Fa w = v * M */

/* Procs de visualitzaci 2D */

void CreaMatTmd (Window w, Viewport vp, mat3x3 M);
/* Crea la matriu de transformaci de mn a dispositiu */

void CreaMatTdm (Window w, Viewport vp, mat3x3 M);
/* Crea la matriu de transformaci de dispositiu a mn */

PtDisp TrMonDisp (Pt2D p, mat3x3 M);
/* Aplica la transf. mn-dispositiu de matriu M al punt p i
retorna un punt en coordenades de dispositiu*/

Pt2D TrDispMon (PtDisp p, mat3x3 M);
/* Aplica la transf. dispositiu-mn de matriu M al punt p de
coordenades de dispositiu i retorna un punt de mn */
