/*****************************************************************
 *	Univers teclat.
 *	Implementa varies funcions per obtenir:
 *    - un punt via un cursor mitjanant el teclat (RequestLocator)
 *    - una opci o codi de tecla (RequestChoice o SampleChoice)
 *****************************************************************/

#include <graphics.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include "libgraf.h"
#include "teclat.h"

#define CursorLength  7
#define CursorFactor  3

static int CursorInc;
static long imatge[(CursorLength+1)*(CursorLength+1)];
static int xact, yact;
static Viewport v;

static void DrawCursor(void);
static void UnDrawCursor(void);
static int ObteCodiTecla( int c );


static void DrawCursor(void)
{
	int x1,y1,x2,y2;

	x1 = max(xact-CursorLength, v.vmin.x);
	y1 = max(yact-CursorLength, v.vmin.y);
	x2 = min(xact+CursorLength, v.vmax.x);
	y2 = min(yact+CursorLength, v.vmax.y);
	getimage(x1, y1, x2, y2, (void *)imatge);
	line(x1, yact, x2, yact);
	line(xact, y1, xact, y2);
}


static void UnDrawCursor(void)
{
	int x1,y1;

	x1 = max(xact-CursorLength, v.vmin.x);
	y1 = max(yact-CursorLength, v.vmin.y);
	putimage(x1, y1, (void *)imatge, COPY_PUT);
}


void IniLocatorTeclat( int x, int y, Viewport vp, int inc)
{
	CursorInc = 1;
	for (;inc>1; inc--)
		CursorInc *= CursorFactor;
	xact = x;
	yact = y;
	v = vp;
}


void RequestLocatorTeclat( int *x, int *y, char *conf)
{
	int end;
/*	char c;	*/

	end = 0;
	DrawCursor();
	while ( !end )
	{
		switch (getch()) {
			case '+':
				if ( CursorInc < 150 )
					CursorInc *= CursorFactor;
				break;
			case '-':
				if ( CursorInc >= CursorFactor )
					CursorInc /= CursorFactor;
				break;
			case ' ':
				*conf = RL_SPACE;
				end = 1;
				break;
			case '\r':
				*conf = RL_RETURN;
				end = 1;
				break;
			case '\033':
				*conf = RL_ESCAPE;
				end = 1;
				break;
			case '\0':
				switch (getch()) {
					case 'P':
					        if (yact < v.vmax.y-CursorInc){
						UnDrawCursor();
						yact += CursorInc;
						DrawCursor();}
						break;
					case 'H':
					        if (yact > v.vmin.y+CursorInc){
						UnDrawCursor();
						yact -= CursorInc;
						DrawCursor();}
						break;
					case 'K':
					        if (xact > v.vmin.x+CursorInc){
						UnDrawCursor();
						xact -= CursorInc;
						DrawCursor();}
						break;
					case 'M':
					        if (xact < v.vmax.x-CursorInc){
						UnDrawCursor();
						xact += CursorInc;
						DrawCursor();}
						break;
				}
		}
	}
	*x = xact;
	*y = yact;
	UnDrawCursor();
}

void RequestChoiceTeclat( int *opc, boolea *status )
{
	unsigned int c;

	c = bioskey(0);
	*opc = ObteCodiTecla(c);
	*status = CERT;
}


void SampleChoiceTeclat( int *opc, boolea *status )
{
	unsigned int c;

	c = bioskey(1);
	if (c) {
		c = bioskey(0);
		*opc = ObteCodiTecla(c);
		*status = CERT;
	} else {
		*status = FALS;
	}
}


static int ObteCodiTecla( int c )
{
	unsigned char c1, c2;

	c1 = c & 0xFF;
	c2 = (c & 0xFF00) >> 8;
	if (isprint(c1))
		return c1;
	else
	switch (c1) {
	case 0x08: /* Backspace */
		return C_BACKSPACE;
	case 0x0D: /* Return */
		return C_RETURN;
	case 0x1B: /* Escape */
		return C_ESCAPE;
	case 0x00:
		switch (c2) {
		case 0x48:   /* fletxa amunt */
			return C_FAMUNT;
		case 0x50:   /* fletxa avall */
			return C_FAVALL;
		case 0x4B:   /* fletxa esquerra */
			return C_FESQ;
		case 0x4D:   /* fletxa dreta */
			return C_FDRETA;
		case 0x73:  /* Ctrl + fletxa esquerra */
			return C_CTRL_FESQ;
		case 0x74:  /* Ctrl + fletxa dreta */
			return C_CTRL_FDRETA;
		case 0x53:  /* Suprimir o Delete */
			return C_DELETE;
		case 0x47:  /* Inicio */
			return C_INICIO;
		case 0x4F:  /* Fin */
			return C_FIN;
		case 0x49:  /* Retroceso Pgina */
			return C_REPAG;
		case 0x51:  /* Avance Pgina */
			return C_AVPAG;
		case 0x3B:  /* Tecla de funci F1 */
			return C_F1;
		case 0x3C:  /* Tecla de funci F2 */
			return C_F2;
		case 0x3D:  /* Tecla de funci F3 */
			return C_F3;
		case 0x3E:  /* Tecla de funci F4 */
			return C_F4;
		case 0x3F:  /* Tecla de funci F5 */
			return C_F5;
		case 0x40:  /* Tecla de funci F6 */
			return C_F6;
		case 0x41:  /* Tecla de funci F7 */
			return C_F7;
		case 0x42:  /* Tecla de funci F8 */
			return C_F8;
		case 0x43:  /* Tecla de funci F9 */
			return C_F9;
		case 0x44:  /* Tecla de funci F10 */
			return C_F10;
		}
	case 0xFF:
		switch (c2) {
		case 0x06:  /* Shift + fletxa amunt */
			return C_SHIFT_FAMUNT;
		case 0x09:  /* Shift + fletxa avall */
			return C_SHIFT_FAVALL;
		case 0x07:  /* Shift + fletxa esquerra */
			return C_SHIFT_FESQ;
		case 0x08:  /* Shift + fletxa dreta */
			return C_SHIFT_FDRETA;
		}
	}
	return 0;
}
